package org.mikgul;

import org.hibernate.Session;

import javax.persistence.*;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;

import org.hibernate.query.Query;

@Entity
@Table(name = "authors")
public class Author {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @Column(name = "first_name", nullable = false)
    private String firstName;

    @Column(name = "last_name", nullable = false)
    private String lastName;

    @Column(name = "date_of_birth")
    private LocalDate dateOfBirth;

    private Boolean alive;

    @OneToMany(mappedBy = "author")
    private List<AuthorBook> authorBooks;

    public Author(String firstName, String lastName, LocalDate dateOfBirth, boolean alive) {
        this.firstName = firstName;
        this.lastName = lastName;
        this.dateOfBirth = dateOfBirth;
        this.alive = alive;
        this.authorBooks = new ArrayList<>();
    }

    public Author(String firstName, String lastName) {
        this.firstName = firstName;
        this.lastName = lastName;
        this.dateOfBirth = null;
        this.alive = null;
        this.authorBooks = new ArrayList<>();
    }

    public Author() {}

    public List<AuthorBook> getAuthorBooks() {
        return authorBooks;
    }

    public void addAuthorBook(AuthorBook authorBook) { this.authorBooks.add(authorBook); }

    public Long getId() {
        return id;
    }

    public String getFirstName() {
        return firstName;
    }

    public String getLastName() {
        return lastName;
    }

    public LocalDate getDateOfBirth() {
        return dateOfBirth;
    }

    public boolean isAlive() {
        return alive;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    public void setDateOfBirth(LocalDate dateOfBirth) {
        this.dateOfBirth = dateOfBirth;
    }

    public void setAlive(boolean alive) {
        this.alive = alive;
    }

    public void setAuthorBooks(List<AuthorBook> authorBooks) {
        this.authorBooks = authorBooks;
    }

    public void save(Session session) {
        session.beginTransaction();
        session.save(this);
        session.getTransaction().commit();
    }

    public static Author getById(Session session, Long authorId) {
        return session.get(Author.class, authorId);
    }

    public void update(Session session) {
        session.beginTransaction();
        session.update(this);
        session.getTransaction().commit();
    }

    public void delete(Session session) {
        session.beginTransaction();
        session.delete(this);
        session.getTransaction().commit();
    }

    public static List<Author> getAuthorsByLastName(Session session, String lastName) {
        Query<Author> query = session.createQuery("FROM Author WHERE last_name = :lastName", Author.class);
        query.setParameter("lastName", lastName);
        return query.getResultList();
    }
}
