package org.mikgul;

import org.hibernate.Session;

import javax.persistence.*;

@Entity
@Table(name = "authors_books")
public class AuthorBook {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @ManyToOne
    @JoinColumn(name = "author_id", referencedColumnName = "id")
    private Author author;

    @ManyToOne
    @JoinColumn(name = "book_id", referencedColumnName = "id")
    private Book book;

    public AuthorBook(Author author, Book book) {
        this.author = author;
        this.book = book;
        author.addAuthorBook(this);
        book.addAuthorBook(this);
    }

    public AuthorBook() {}

    public Long getId() {
        return id;
    }

    public Author getAuthor() {
        return author;
    }

    public Book getBook() {
        return book;
    }

    public void setAuthor(Author author) {
        this.author = author;
    }

    public void setBook(Book book) {
        this.book = book;
    }

    public void save(Session session) {
        session.beginTransaction();
        session.save(this);
        session.getTransaction().commit();
    }

    public static AuthorBook getById(Session session, Long authorBookId) {
        return session.get(AuthorBook.class, authorBookId);
    }

    public void update(Session session) {
        session.beginTransaction();
        session.update(this);
        session.getTransaction().commit();
    }

    public void delete(Session session) {
        session.beginTransaction();
        session.delete(this);
        session.getTransaction().commit();
    }
}
