package org.mikgul;

import org.hibernate.Session;

import javax.persistence.*;
import java.time.ZonedDateTime;
import java.util.List;

@Entity
public class Bestseller {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @OneToOne(cascade = CascadeType.PERSIST)
    @JoinColumn(name = "book_id", referencedColumnName = "id")
    private Book book;

    @Column(name = "copies_sold", nullable = false)
    private long copiesSold;

    @Column(name = "rating", nullable = false)
    private short rating;

    @Column(name = "movie_adaptation")
    private Boolean movieAdaptation;

    @Column(name = "release_date")
    private ZonedDateTime releaseDate;

    private Integer awards;

    public Bestseller(Book book, long copiesSold, short rating, Boolean movieAdaptation, ZonedDateTime releaseDate, Integer awards) {
        this.book = book;
        this.copiesSold = copiesSold;
        this.rating = rating;
        this.movieAdaptation = movieAdaptation;
        this.releaseDate = releaseDate;
        this.awards = awards;
    }

    public Bestseller(Book book, long copiesSold, short rating) {
        this.book = book;
        this.copiesSold = copiesSold;
        this.rating = rating;
        this.movieAdaptation = null;
        this.releaseDate = null;
        this.awards = null;
    }

    public Bestseller() {}

    public Long getId() {
        return id;
    }

    public ZonedDateTime getReleaseDate() {
        return releaseDate;
    }

    public Book getBook() {
        return book;
    }

    public long getCopiesSold() {
        return copiesSold;
    }

    public int getRating() {
        return rating;
    }

    public Boolean hasMovieAdaptation() {
        return movieAdaptation;
    }

    public Integer getAwards() {
        return awards;
    }

    public void setBook(Book book) {
        this.book = book;
    }

    public void setCopiesSold(long copiesSold) {
        this.copiesSold = copiesSold;
    }

    public void setRating(short rating) {
        this.rating = rating;
    }

    public void setMovieAdaptation(boolean movieAdaptation) {
        this.movieAdaptation = movieAdaptation;
    }

    public void setReleaseDate(ZonedDateTime releaseDate) {
        this.releaseDate = releaseDate;
    }

    public void setAwards(Integer awards) {
        this.awards = awards;
    }

    public void save(Session session) {
        session.beginTransaction();
        session.save(this);
        session.getTransaction().commit();
    }

    public static Bestseller getById(Session session, Long bestsellerId) {
        return session.get(Bestseller.class, bestsellerId);
    }

    public void update(Session session) {
        session.beginTransaction();
        session.update(this);
        session.getTransaction().commit();
    }

    public void delete(Session session) {
        session.beginTransaction();
        session.delete(this);
        session.getTransaction().commit();
    }

    public static List<Bestseller> getBestsellersWithHighRatings(Session session, short minRating) {
        if (minRating < 0 || minRating > 100) {
            System.err.println("Rating must be a number between 0-100.");
            return null;
        }
        return session.createQuery("FROM Bestseller b WHERE b.rating >= :minRating", Bestseller.class)
                .setParameter("minRating", minRating)
                .getResultList();
    }
}
