package org.mikgul;

import org.hibernate.Session;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.List;

@Entity
@Table(name = "books")
public class Book {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @Column(nullable = false)
    private String title;

    private boolean bestseller;

    @ManyToOne
    @JoinColumn(name = "genre_id", referencedColumnName = "id")
    private Genre genre;

    @OneToMany(mappedBy = "book")
    private List<AuthorBook> authorBooks;

    public List<AuthorBook> getAuthorBooks() {
        return authorBooks;
    }

    public Book(String title, boolean bestseller, Genre genre) {
        this.title = title;
        this.bestseller = bestseller;
        this.genre = genre;
        this.authorBooks = new ArrayList<AuthorBook>();
    }

    public Book() {}

    public void addAuthorBook(AuthorBook authorBook) { this.authorBooks.add(authorBook); }

    public Long getId() {
        return id;
    }

    public String getTitle() {
        return title;
    }

    public boolean isBestseller() {
        return bestseller;
    }

    public Genre getGenre() {
        return genre;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public void setBestseller(boolean bestseller) {
        this.bestseller = bestseller;
    }

    public void setGenre(Genre genre) {
        this.genre = genre;
    }

    public void setAuthorBooks(List<AuthorBook> authorBooks) {
        this.authorBooks = authorBooks;
    }

    public void save(Session session) {
        session.beginTransaction();
        session.save(this);
        session.getTransaction().commit();
    }

    public static Book getById(Session session, Long bookId) {
        return session.get(Book.class, bookId);
    }

    public void update(Session session) {
        session.beginTransaction();
        session.update(this);
        session.getTransaction().commit();
    }

    public void delete(Session session) {
        session.beginTransaction();
        session.delete(this);
        session.getTransaction().commit();
    }

    public static List<Book> getBooksPaginated(Session session, int pageNumber, int pageSize) {
        if (pageNumber < 0 || pageSize < 0) {
            System.err.println("Page number and page size cannot be negative.");
            return null;
        }
        return session.createQuery("FROM Book", Book.class)
                .setFirstResult((pageNumber - 1) * pageSize)
                .setMaxResults(pageSize)
                .getResultList();
    }
}
