package org.mikgul;

import org.hibernate.Session;

import javax.persistence.*;

import java.time.LocalDate;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.List;

public class Main {

    public static void main(String[] args) {
        // przykladowe dane
        Author author1 = new Author("John", "Doe", LocalDate.of(1962, 2, 1), true);
        Author author2 = new Author("Mary", "Sue", LocalDate.of(1954, 5, 21), false);
        Author author3 = new Author("Joe", "Doe", LocalDate.of(1828, 11, 29), false);

        Genre genre1 = new Genre("Fantasy");
        Genre genre2 = new Genre("Historical");

        Book book1 = new Book("Famous book", true, genre1);
        Book book2 = new Book("Not-so-famous book", false, genre2);
        Book book3 = new Book("Very famous book", true, genre2);
        Book book4 = new Book("Old and famous book", true, genre1);

        Bestseller bestseller1 = new Bestseller(book1, 92147823L, (short) 72);
        Bestseller bestseller2 = new Bestseller(book4, 1230589251, (short) 76);
        Bestseller bestseller3 = new Bestseller(book3,
                105134113L,
                (short) 91,
                true,
                ZonedDateTime.of(1990, 4, 5, 9, 0, 0, 0, ZoneId.of("+02:00")),
                4);

        AuthorBook authorBook1 = new AuthorBook(author1, book1);
        AuthorBook authorBook2 = new AuthorBook(author2, book1);
        AuthorBook authorBook3 = new AuthorBook(author2, book2);
        AuthorBook authorBook4 = new AuthorBook(author1, book3);
        AuthorBook authorBook5 = new AuthorBook(author3, book4);

        // Zapytania (Queries)
        try {
            EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("hibernate-dynamic");

            EntityManager entityManager = entityManagerFactory.createEntityManager();
            Session session = entityManager.unwrap(Session.class);

            // zapisuje przykladowe obiekty do bazy
            session.beginTransaction();
            session.save(author1);
            session.save(author2);
            session.save(author3);
            session.save(genre1);
            session.save(genre2);
            session.save(book1);
            session.save(book2);
            session.save(book3);
            session.save(book4);
            session.save(bestseller1);
            session.save(bestseller2);
            session.save(bestseller3);
            session.save(authorBook1);
            session.save(authorBook2);
            session.save(authorBook3);
            session.save(authorBook4);
            session.save(authorBook5);
            session.getTransaction().commit();

            // pokazuje bestsellery z ocena przynajmniej tak wysoka jak podana
            System.out.println("===Informacje o bestsellerach z oceną przynajmniej 75/100===");
            List<Bestseller> bestsellers = Bestseller.getBestsellersWithHighRatings(session, (short) 75);
            if (bestsellers != null) {
                for (Bestseller b : bestsellers) {
                    System.out.printf("Title: %s\nRelease date: %s\nRating: %d/100\nAwards: %d\n\n",
                            b.getBook().getTitle(),
                            b.getReleaseDate(),
                            b.getRating(),
                            b.getAwards());
                }
            }

            // pokazuje informacje o autorach z podanym nazwiskiem
            System.out.println("===Informacje o autorach z nazwiskiem `Doe`===");
            for (Author a : Author.getAuthorsByLastName(session, "Doe")) {
                System.out.printf("First name: %s\nDate of birth: %s\nAlive: %b\nBooks written: \n",
                        a.getFirstName(), a.getDateOfBirth(), a.isAlive());
                for (AuthorBook ab : a.getAuthorBooks()) {
                    System.out.printf("  - %s\n", ab.getBook().getTitle());
                }
                System.out.println();
            }

            // pokazuje ksiazki (metoda stronicowana)
            System.out.println("===Lista książek (stronicowane)===");
            List<Book> books = Book.getBooksPaginated(session, 1, 3);
            if (books != null) {
                for (Book b : books) {
                    System.out.printf("\nTitle: %s\nGenre: %s\nIs bestseller: %b\nAuthors:\n",
                            b.getTitle(), b.getGenre().getName(), b.isBestseller());
                    for (AuthorBook ab : b.getAuthorBooks()) {
                        System.out.printf("  - %s %s\n", ab.getAuthor().getFirstName(), ab.getAuthor().getLastName());
                    }
                }
            }
            session.close();
        } catch (Throwable ex) {
            System.err.println("Creating session failed." + ex);
        }
    }
}
