import org.hibernate.Session;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

import org.mikgul.Genre;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

public class TestGenre {
    private Session session;

    @Before
    public void setUp() {
        try {
            EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory("hibernate-dynamic");

            EntityManager entityManager = entityManagerFactory.createEntityManager();
            this.session = entityManager.unwrap(Session.class);
        } catch (Throwable ex) {
            System.err.println("Creating session failed." + ex);
        }
    }

    @After
    public void tearDown() {
        if (this.session != null) {
            this.session.close();
        }
    }

    @Test
    public void testCRUD() {
        Genre genre = new Genre("Fantasy");

        // CREATE
        genre.save(session);
        assertNotNull(genre.getId());

        // READ
        Genre retrievedGenre = Genre.getById(session, genre.getId());
        assertNotNull(retrievedGenre);
        assertEquals(genre.getName(), retrievedGenre.getName());

        // UPDATE
        retrievedGenre.setName("Dark-Fantasy");
        retrievedGenre.update(session);
        Genre updatedGenre = Genre.getById(session, retrievedGenre.getId());
        assertNotNull(updatedGenre);
        assertEquals("Dark-Fantasy", updatedGenre.getName());

        // DELETE
        genre.delete(session);
        Genre deletedGenre = Genre.getById(session, genre.getId());
        assertNull(deletedGenre);
    }
}